@@ -8,15 +8,18 @@ Created by Maxim Ziatdinov (email: maxim.ziatdinov@ai4microscopy.com)
 """
 
 import os
-import numpy as np
-from sklearn import mixture, decomposition, cluster
-from scipy import spatial
-from atomai.utils import get_nn_distances, get_intensities
-import matplotlib.pyplot as plt
-import matplotlib.gridspec as gridspec
-from matplotlib import cm
 import warnings
 
+import matplotlib.gridspec as gridspec
+import matplotlib.pyplot as plt
+import numpy as np
+from matplotlib import cm
+from scipy import spatial
+from sklearn import cluster, decomposition, mixture
+
+from atomai.utils import (get_intensities, get_nn_distances, peak_refinement,
+                          plot_transitions)
+
 
 class imlocal:
     """
@@ -27,20 +30,23 @@ class imlocal:
 
     Args:
         network_output (4D numpy array):
-            Batch_size x height x width x channels
+            Output of a fully convolutional neural network where
+            a class is assigned to every pixel in the input image(s).
+            The dimensions are
+            :math:`images \\times height \\times width \\times channels`
         coord_class_dict_all (dict):
             Prediction from atomnet.locator
-            (can be from other but must be the same format)
-            Each element is a N x 3 numpy array,
-            where N is a number of detected atoms/defects,
-            the first 2 columns are xy coordinates
+            (can be from other source but must be in the same format)
+            Each element is a :math:`N \\times 3` numpy array,
+            where *N* is a number of detected atoms/defects,
+            the first 2 columns are *xy* coordinates
             and the third columns is class (starts with 0)
         crop_size (int):
             Half of the side of the square for subimage cropping
         coord_class (int):
             Class of atoms/defects around around which
             the subimages will be cropped; in the atomnet.locator output
-            the class is the 3rd column (the first two are xy positions)
+            the class is the 3rd column (the first two are *xy* positions)
 
     Examples:
 
@@ -91,19 +97,11 @@ class imlocal:
         Extracts subimages centered at certain atom class/type
         in the neural network output
 
-        Args:
-            imgdata (4D numpy array):
-                Prediction of a neural network with dimensions
-                (batch_size x height x width x channels)
-            coord (N x 3 numpy array):
-                (x, y, class) coordinates data
-            d (int):
-                Defines size of a square subimage
-
         Returns:
-            stack of subimages,
-            (x, y) coordinates of their centers,
-            frame number associated with each subimage
+            3-element tuple containing
+            i) stack of subimages,
+            ii) (x, y) coordinates of their centers,
+            iii) frame number associated with each subimage
         """
         imgstack, imgstack_com, imgstack_frames = [], [], []
         for i, (img, coord) in enumerate(
@@ -129,15 +127,16 @@ class imlocal:
         Args:
             imgdata (3D numpy array):
                 Prediction of a neural network with dimensions
-                (height x width x channels)
+                :math:`height \\times width \\times n channels`
             coord (N x 2 numpy array):
                 (x, y) coordinates
             r (int):
-                Square subimage side is 2*r
+                Half side of subimage square
 
         Returns:
-            stack of subimages and
-            (x, y) coordinates of their centers
+            2-element tuple containing
+            i) Stack of subimages and
+            ii) (x, y) coordinates of their centers
         """
         img_cr_all = []
         com = []
@@ -174,13 +173,13 @@ class imlocal:
                 Plotting gmm components
 
         Returns:
-            4D numpy array containing averaged images for each gmm class
+            3-element tuple containing
+            i) 4D numpy array (*n_components* x *height* x *width* x *channels*)
+            containing averaged images for each GMM class
             (the 1st dimension correspond to individual mixture components),
-            List where each element contains images from the self.imgstack
-            (as 4D numpy array) belonging to each GMM class,
-            4D numpy array with xy coordinates of the center of mass
-            for each subimage from the stack, labels for every subimage
-            and a frame number for each label
+            ii) List where each element contains 4D images belonging to each GMM class,
+            iii) 2D numpy array (*N* x 4) with *xy* coordinates of the center of mass,
+            labels and a frame number for each subimage
         """
         clf = mixture.GaussianMixture(
             n_components=n_components,
@@ -225,8 +224,7 @@ class imlocal:
             random_state=1,
             plot_results=False):
         """
-        Computes PCA eigenvectors and their loading maps
-        for a stack of subimages.
+        Computes PCA eigenvectors for a stack of subimages.
 
         Args:
             n_components (int):
@@ -237,12 +235,13 @@ class imlocal:
                 Plots computed eigenvectors
 
         Returns:
-            4D numpy array with computed (and reshaped) principal axes
-            for stack of subimages,
-            2D numpy array with projection of X_vec
+            3-element tuple containing
+            i) 4D numpy array (*n_components* x *height* x *width* x *channels*)
+            with computed (and reshaped) principal axes for stack of subimages,
+            ii) 2D numpy array (*N* x *n_components*) with projection of X_vec
             on the first principal components,
-            3D numpy array with center-of-mass coordinates
-            and the corresponding label number for each subimage
+            iii) 2D numpy array (*N* x 3) with center-of-mass coordinates
+            and the corresponding frame number for each subimage
         """
         pca = decomposition.PCA(
             n_components=n_components,
@@ -263,8 +262,7 @@ class imlocal:
             random_state=1,
             plot_results=False):
         """
-        Computes ICA independent souces and their loading maps
-        for a stack of subimages.
+        Computes ICA independent souces for a stack of subimages.
 
         Args:
             n_components (int):
@@ -275,12 +273,14 @@ class imlocal:
                 Plots computed sources
 
         Returns:
-            4D numpy array with computed (and reshaped) independent sources
+            3-element tuple containing
+            i) 4D numpy array (*n_components* x *height* x *width* x *channels*)
+            with computed (and reshaped) independent sources
             for stack of subimages,
-            2D numpy array with recovered sources
-            from X_vec,
-            3D numpy array with center-of-mass coordinates
-            and the corresponding label number for each subimage
+            ii) 2D numpy array (*N* x *n_components*) numpy array with
+            recovered sources from X_vec,
+            iii) 2D numpy aray (*N* x 3) with center-of-mass coordinates
+            and the corresponding frame number for each subimage
         """
         ica = decomposition.FastICA(
             n_components=n_components,
@@ -302,11 +302,7 @@ class imlocal:
             plot_results=False,
             **kwargs):
         """
-        Applies NMF to source separation.
-        Computes sources and their loading maps
-        for a stack of subimages. Intended to be used for
-        finding domains ("blocks") (e.g. ferroic domains)
-        in a single image.
+        Applies NMF to source separation from a stack of subimages
 
         Args:
             n_components (int):
@@ -319,12 +315,13 @@ class imlocal:
                 Maximum number of iterations before timing out
 
         Returns:
-            4D numpy array with computed (and reshaped) sources
-            for stack of subimages,
-            2D numpy array with transformed data X_vec according
-            to the trained NMF model,
-            3D numpy array with center-of-mass coordinates
-            and the corresponding label number for each subimage
+            3-element tuple containing
+            i) 4D numpy array (*n_components* x *height* x *width* x *channels*)
+            with computed (and reshaped) sources for stack of subimages,
+            ii) 2D numpy array (*N* x *n_components*)  with
+            transformed data X_vec according to the trained NMF model,
+            iii) 2D numpy array(*N* x 3) with center-of-mass coordinates
+            and the corresponding frame number for each subimage
         """
 
         max_iter = kwargs.get('max_iterations', 1000)
@@ -368,11 +365,13 @@ class imlocal:
                 Plotting GMM components
 
         Returns:
-            4D numpy array containing averaged images for each gmm class
-            (the 1st dimension correspond to individual mixture components),
-            List of 4D numpy arrays with PCA components,
-            List of PCA-transformed data,
-            4D numpy array with xy coordinates of the center of mass
+            4-element tuple containing
+            i) 4D numpy array (*n_components_gmm* x *height* x *width* x *channels*)
+            containing averaged images for each gmm class,
+            ii) List of 4D numpy arrays with PCA components
+            (*n_components_pca* x *height* x *width* x *channels*),
+            iii) List of PCA-transformed data,
+            iv) 2D numpy array (*N* x 4) with *xy* coordinates of the center of mass
             for each subimage from the stack used for GMM, GMM-assigned label
             for every subimage and a frame number for each label
         """
@@ -478,9 +477,11 @@ class imlocal:
                 Controls marker size for loading maps plot
 
         Returns:
-            4D numpy array with computed (and reshaped) principal axes
-            for stack of subimages and 2D numpy array with projection of X_vec
-            on the first principal components
+            2-element tuple containing
+            i) 4D numpy array (*n_components* x *height* x *width* x *channels*)
+            with computed (and reshaped) principal axes
+            for stack of subimages and ii) 2D numpy array (*N* x *n_components*)
+            with projection of X_vec on the first principal components
         """
 
         m_s = kwargs.get('marker_size')
@@ -517,9 +518,11 @@ class imlocal:
                 controls marker size for loading maps plot
 
         Returns:
-            4D numpy array with computed (and reshaped) independent sources
-            for stack of subimages and 2D numpy array with recovered sources
-            from X_vec
+            2-element tuple containing
+            i) 4D numpy array (*n_components* x *height* x *width* x *channels*)
+            with computed (and reshaped) independent sources
+            for stack of subimages and ii) 2D numpy array (*N* x *n_components*)
+            with recovered sources from X_vec
         """
 
         m_s = kwargs.get('marker_size')
@@ -559,9 +562,11 @@ class imlocal:
                 Controls marker size for loading maps plot
 
         Returns:
-            4D numpy array with computed (and reshaped) sources
-            for stack of subimages and 2D numpy array with
-            transformed data X_vec according to the trained NMF model
+            2-element tuple containing
+            i) 4D numpy array (*n_components* x *height* x *width* x *channels*)
+            with computed (and reshaped) sources
+            for stack of subimages and ii) 2D numpy array (*N* x *n_components*)
+            with transformed data X_vec according to the trained NMF model
         """
 
         m_s = kwargs.get('marker_size')
@@ -659,8 +664,9 @@ class imlocal:
                 in the next one
 
         Returns:
-            Numpy array of defect/atom coordinaes form a single trajectory
-            and frames corresponding to this trajectory
+            2-element tuple containing
+            i) Numpy array of defect/atom coordinates form a single trajectory
+            and ii) frames corresponding to this trajectory
         """
         flow = np.empty((0, 3))
         frames = []
@@ -702,14 +708,16 @@ class imlocal:
                 Random state instance for Gaussian mixture model
 
         Returns:
-            list defects/atoms trajectories (each trajectory is numpy array),
-            list of frames corresponding to the extracted trajectories
+            Python dictionary containing
+            i) list of numpy arrays with defects/atoms trajectories ("trajectories"),
+            ii) list of frames corresponding to the extracted trajectories ("frames"),
+            iii) gmm components when run_gmm=True ("gmm_components")
         """
         if run_gmm:
             n_components = kwargs.get("n_components", 5)
             covariance = kwargs.get("covariance", "diag")
             random_state = kwargs.get("random_state", 1)
-            _, _, classes = self.gmm(
+            gmm_comps, _, classes = self.gmm(
                 n_components, covariance, random_state)
             classes = classes[:, -2]
         else:
@@ -728,7 +736,11 @@ class imlocal:
             if len(flow) > min_length:
                 all_trajectories.append(flow)
                 all_frames.append(frames)
-        return all_trajectories, all_frames
+        return_dict = {"trajectories": all_trajectories,
+                       "frames": all_frames}
+        if run_gmm:
+            return_dict["gmm_components"] = gmm_comps
+        return return_dict
 
     @classmethod
     def renumerate_classes(cls, classes):
@@ -746,7 +758,8 @@ class imlocal:
                           covariance='diag',
                           random_state=1,
                           rmax=10,
-                          min_length=0):
+                          min_length=0,
+                          sum_all_transitions=False):
         """
         Applies Gaussian mixture model to a stack of
         local descriptors (subimages). Extracts trajectories for
@@ -769,86 +782,79 @@ class imlocal:
                 Minimal length of trajectory to return
 
         Returns:
-            list defects/atoms trajectories,
-            list of transition matrices for each trajectory,
-            list of frames corresponding to the extracted trajectories
+            Pyhton dictionary containing
+            i) list of defects/atoms trajectories ("trajectories"),
+            ii) list of transition matrices for each trajectory ("transitions"),
+            iii) list of frames corresponding to the extracted trajectories ("frames"),
+            iv) GMM components as images ("gmm_components")
         """
-        trajectories_all, frames_all = self.get_all_trajectories(
-            min_length, run_gmm=True, n_components=n_components,
-            covariance=covariance, random_state=random_state, rmax=rmax)
+        dict_to_return = self.get_all_trajectories(
+            min_length, run_gmm=True, n_components=n_components, rmax=rmax,
+            covariance=covariance, random_state=random_state)
         transitions_all = []
-        for traj in trajectories_all:
+        for traj in dict_to_return["trajectories"]:
             classes = self.renumerate_classes(traj[:, -1])
-            m = transitions(classes).calculate_transition_matrix()
+            m = calculate_transition_matrix(classes)
             transitions_all.append(m)
-        return trajectories_all, transitions_all, frames_all
+        dict_to_return["transitions"] = transitions_all
+        if sum_all_transitions:
+            dict_to_return["all_transitions"] = sum_transitions(
+                dict_to_return, n_components)
+        return dict_to_return
 
 
-class transitions:
+def calculate_transition_matrix(trace):
     """
-    Calculates and displays (optionally) Markov transition matrix
+    Calculates Markov transition matrix
 
     Args:
         trace (1D numpy array or python list):
             sequence of states/classes
     """
-    def __init__(self, trace):
-        self.trace = trace
+    n = 1 + max(trace)  # number of states
+    M = np.zeros(shape=(n, n))
+    for (i, j) in zip(trace, trace[1:]):
+        M[i][j] += 1
+    # convert to probabilities:
+    for row in M:
+        s = sum(row)
+        if s > 0:
+            row[:] = [f/s for f in row]
+    return M
 
-    def calculate_transition_matrix(self,
-                                    plot_results=False,
-                                    plot_values=False):
-        """
-        Calculates Markov transition matrix
 
-        Args:
-            plot_results (bool):
-                Plot calculated transition matrix
-            plot_values (bool):
-                Show calculated transition rates
+def sum_transitions(trans_dict, msize, plot_results=False, **kwargs):
+    """
+    Sums and normalizes transitions associated with individual trajectories
 
-        Returns:
-            Calculated transition matrix as 2D numpy array
-        """
-        n = 1 + max(self.trace) # number of states
-        M = np.zeros(shape=(n, n))
-        for (i, j) in zip(self.trace, self.trace[1:]):
-            M[i][j] += 1
-        # now convert to probabilities:
-        for row in M:
-            s = sum(row)
-            if s > 0:
-                row[:] = [f/s for f in row]
-        if plot_results:
-            self.plot_transition_matrix(M, plot_values)
-        return M
-
-    @classmethod
-    def plot_transition_matrix(cls, m, plot_values=False):
-        """
-        Plots transition matrix
-
-        Args:
-            m (2D numpy array):
-                Transition matrix
-            plot_values (bool):
-                Show calculated transtion rates
-        """
-        print('Transition matrix')
-        _, ax = plt.subplots(1, 1, figsize=(6, 6))
-        ax.matshow(m, cmap='Reds')
-        xt = np.arange(len(m))
-        yt = np.arange(len(m))
-        ax.set_xticks(xt)
-        ax.set_yticks(yt)
-        ax.set_xticklabels((xt+1).tolist(), rotation='horizontal', fontsize=14)
-        ax.set_yticklabels((yt+1).tolist(), rotation='horizontal', fontsize=14)
-        ax.set_title('Transition matrix', y=1.1, fontsize=20)
-        for (j, i), v in np.ndenumerate(m):
-            ax.text(i, j, np.around(v, 2), ha='center', va='center', c='b')
-        ax.set_xlabel('Transition class', fontsize=18)
-        ax.set_ylabel('Starting class', fontsize=18)
-        plt.show()
+    Args:
+        trans_dict (dict):
+            Python dictionary containing trajectories, frame numbers,
+            transitions and the averaged GMM components. Usually this is
+            an output of atomstat.transition_matrix
+        msize (int):
+            (m, m) size of full transition matrix
+        plot_results (bool):
+            plot transition matrix and GMM components
+            associated with highest transition frequencies
+        **transitions_to_plot (int):
+            number of transitions (associated with largerst prob values) to plot
+    
+    Returns:
+        Full transition matrix as 2D numpy array
+    """
+    transmat_all = np.zeros((msize, msize))
+    for traj, trans in zip(trans_dict["trajectories"], trans_dict["transitions"]):
+        states = np.unique(traj[:, -1]).astype(np.int64)
+        for (i, j), v in np.ndenumerate(trans):
+            transmat_all[states[i]-1, states[j]-1] += v
+    transmat_all = transmat_all/transmat_all.sum(axis=1, keepdims=1)
+    if plot_results:
+        plot_transitions(
+            transmat_all, 
+            gmm_components=trans_dict["gmm_components"],
+            **kwargs)
+    return transmat_all
 
 
 def plot_lattice_bonds(distances,
@@ -966,9 +972,9 @@ def cluster_coord(coord_class_dict, eps, min_samples=10):
 
     Args:
         coord_class_dict (dict):
-            Dictionary of atomic coordinates (N x 3 numpy arrays])
+            Dictionary of atomic coordinates (:math:`N \\times 3` numpy arrays])
             (same format as produced by atomnet.locator)
-            Can also be a list of N x 3 numpy arrays
+            Can also be a list of :math:`N \\times 3` numpy arrays
             Typically, these are coordinates from a 3D image stack
             where each element in dict/list corresponds
             to an individual movie frame
@@ -980,9 +986,10 @@ def cluster_coord(coord_class_dict, eps, min_samples=10):
             Minmum number of points for a "cluster"
 
     Returns:
-        Coordinates of points in each identified cluster,
-        center of the mass for each cluster,
-        standard deviation of points in each cluster
+        3-element tuple containing
+        i) coordinates of points in each identified cluster,
+        ii) center of the mass for each cluster,
+        iii) standard deviation of points in each cluster
     """
     coordinates_all = np.empty((0, 3))
     for k in range(len(coord_class_dict)):
@@ -1017,7 +1024,8 @@ def find_coord_clusters(coord_class_dict_1, coord_class_dict_2, rmax):
             (usually from an output of atomnet.predictor for a single image;
             can be from other source but should be in the same format)
         coord_class_dict_2 (dict or list):
-            Dictionary or list of atomic coordinates (N x 3 numpy arrays)
+            Dictionary or list of atomic coordinates
+            (:math:`N \\times 3` numpy arrays)
             These can be coordinates from a 3D image stack
             where each element in dict/list corresponds
             to an individual frame in the stack.
@@ -1027,9 +1035,10 @@ def find_coord_clusters(coord_class_dict_1, coord_class_dict_2, rmax):
             Maximum search radius in pixels
 
     Returns:
-        Coordinates of points in each identified cluster,
-        center of the mass for each cluster,
-        standard deviation of points in each cluster
+        3-element tuple containing
+        i) coordinates of points in each identified cluster,
+        ii) center of the mass for each cluster,
+        iii) standard deviation of points in each cluster
     """
     coordinates_all = np.empty((0, 3))
     for k in range(len(coord_class_dict_2)):
@@ -1105,3 +1114,28 @@ def update_classes(coordinates,
         raise NotImplementedError(
             "Choose between 'threshold' and 'kmeans' methods")
     return coordinates
+
+
+def update_positions(coordinates, nn_input, d=None):
+    """
+    Updates atomic/defect coordinates based on
+    peak refinement procedure at each predicted position
+
+    Args:
+        coordinates (dict):
+            Output of atomnet.predictor
+        nn_input (numpy array):
+            Image(s) served as an input to neural network
+        d (int):
+        Half of the side of the square box where the fitting is performed;
+        defaults to 1/4 of mean nearest neighbor distance in the system
+
+    Returns:
+        Updated coordinates
+    """
+    print('\rRefining atomic positions... ', end="")
+    coordinates_r = {}
+    for i, (img, coord) in enumerate(zip(nn_input, coordinates.values())):
+        coordinates_r[i] = peak_refinement(img[..., 0], coord, d)
+    print("Done")
+    return coordinates_r
