@@ -148,10 +148,12 @@ def preprocess_training_data(images_all,
         raise AssertionError(
             "Provide all training and test data in the same format")
     if isinstance(labels_all, list):
-        num_classes = max(set([len(np.unique(lab)) for lab in labels_all]))
+        pass
     elif isinstance(labels_all, dict):
-        num_classes = max(
-            set([len(np.unique(lab)) for lab in labels_all.values()]))
+        images_all = [i for i in images_all.values()]
+        labels_all = [i for i in labels_all.values()]
+        images_test_all = [i for i in images_test_all.values()]
+        labels_test_all = [i for i in labels_test_all.values()]
     elif isinstance(labels_all, np.ndarray):
         n_train_batches, _ = np.divmod(labels_all.shape[0], batch_size)
         n_test_batches, _ = np.divmod(labels_test_all.shape[0], batch_size)
@@ -163,13 +165,13 @@ def preprocess_training_data(images_all,
             images_test_all[:n_test_batches*batch_size], n_test_batches)
         labels_test_all = np.split(
             labels_test_all[:n_test_batches*batch_size], n_test_batches)
-        num_classes = max(set([len(np.unique(lab)) for lab in labels_all]))
     else:
         raise NotImplementedError(
             "Provide training and test data as python list (or dictionary)",
             "of numpy arrays or as 4D (images)",
             "and 4D/3D (labels for single/multi class) numpy arrays"
         )
+    num_classes = max(set([len(np.unique(lab)) for lab in labels_all]))
     if num_classes == 1:
         raise AssertionError(
             "Confirm that you have a class corresponding to background")
@@ -270,14 +272,14 @@ def img_resize(image_data, rs):
 def img_pad(image_data, pooling):
     """
     Pads the image if its size (w, h)
-    is not divisible by 2**n, where n is a number
+    is not divisible by :math:`2^n`, where n is a number
     of pooling layers in a network
 
     Args:
         image_data (3D numpy array):
             Image stack with dimensions (n_batches x height x width)
         pooling (int):
-            Downsampling factor (equal to 2**n, where n is a number
+            Downsampling factor (equal to :math:`2^n`, where *n* is a number
             of pooling operations)
     """
     # Pad image rows (height)
@@ -327,8 +329,8 @@ def get_nn_distances_(coordinates, nn=2, upper_bound=None):
             Upper distance bound for Query the kd-tree for nearest neighbors.
             Only di
     Returns:
-        Tuple with :math:`n atoms \\times nn` array of distances to nearest
-        neighbors and :math:`n atoms \\times (nn+1) \\times 3` array of coordinates
+        Tuple with :math:`atoms \\times nn` array of distances to nearest
+        neighbors and :math:`atoms \\times (nn+1) \\times 3` array of coordinates
         (including coordinates of the "center" atom), where n_atoms is less or
         equal to the total number of atoms in the 'coordinates'
         (due to 'upper_bound' criterion)
@@ -358,8 +360,8 @@ def get_nn_distances(coordinates, nn=2, upper_bound=None):
             Upper distance bound for Query the kd-tree for nearest neighbors.
             Only distances below this value will be counted.
     Returns:
-        Tuple with list of :math:`n_atoms \\times nn` arrays of distances
-        to nearest neighbors and list of :math:`n_atoms \\times (nn+1) \\times 3`
+        Tuple with list of :math:`atoms \\times nn` arrays of distances
+        to nearest neighbors and list of :math:`atoms \\times (nn+1) \\times 3`
         array of coordinates (including coordinates of the "center" atom),
         where n_atoms is less or equal to the total number of atoms in the
         'coordinates' (due to 'upper_bound' criterion)
@@ -673,6 +675,146 @@ def draw_boxes(imgdata, defcoord, bbox=16, fsize=6):
     plt.show()
 
 
+def plot_trajectories(traj, frames, **kwargs):
+    """
+    Plots individual trajectory (as position (radius) vector)
+
+    Args:
+        traj (n x 3 ndarray):
+            numpy array where first two columns are coordinates
+            and the 3rd columd are classes
+        frames ((n,) ndarray):
+            numpy array with frame numbers
+        **fov (int or list):
+            field of view or scan size
+        **fsize (int):
+            figure size
+        **cmap (str):
+            colormap (default: jet)
+    """
+    fov = kwargs.get("fov")
+    cmap = kwargs.get("cmap", "jet")
+    fsize = kwargs.get("fsize", 6)
+    r_coord = np.linalg.norm(traj[:, :2], axis=1)
+    plt.figure(figsize=(fsize*2, fsize))
+    plt.scatter(frames, r_coord, c=traj[:, -1], cmap=cmap)
+    if fov:
+        if isinstance(fov, list) and len(fov) == 2:
+            fov = np.sqrt(fov[0]**2 + fov[1]**2)
+        elif isinstance(fov, int):
+            fov = np.sqrt(2*fov**2)
+        else:
+            raise ValueError("Pass 'fov' argument as integer or 2-element list")
+        plt.ylim(0, fov)
+    plt.xlabel("Time step (a.u.)", fontsize=18)
+    plt.ylabel("Position vector", fontsize=18)
+    cbar = plt.colorbar()
+    cbar.set_label("States", fontsize=16)
+    plt.clabel
+    plt.title("Trajectory", fontsize=20)
+    plt.show()
+
+
+def plot_transitions(matrix,
+                     states=None,
+                     gmm_components=None,
+                     plot_values=False,
+                     **kwargs):
+    """
+    Plots transition matrix and (optionally) most frequent/probable transitions
+
+    Args:
+        m (2D numpy array):
+            Transition matrix
+        gmm_components (4D numpy array):
+            GMM components (optional)
+        plot_values (bool):
+            Show calculated transtion rates
+        **transitions_to_plot (int):
+            number of transitions (associated with largest prob values) to plot
+        **plot_toself (bool):
+            Skips transitions into self when plotting transitions with largest probs
+        **fsize (int): figure size
+        **cmap (str): color map
+    """
+    fsize = kwargs.get("fsize", 6)
+    cmap = kwargs.get("cmap", "Reds")
+    transitions_to_plot = kwargs.get("transitions_to_plot", 6)
+    plot_toself = kwargs.get("plot_toself", True)
+    m = matrix
+    _, ax = plt.subplots(1, 1, figsize=(fsize, fsize))
+    ax.matshow(m, cmap=cmap)
+    if states is None:
+        states = np.arange(len(m)) + 1
+    xt = states
+    ax.set_xticks(np.arange(len(xt)))
+    ax.set_yticks(np.arange(len(xt)))
+    ax.set_xticklabels((xt).tolist(), rotation='horizontal', fontsize=14)
+    ax.set_yticklabels((xt).tolist(), rotation='horizontal', fontsize=14)
+    ax.set_title('Transition matrix', y=1.1, fontsize=20)
+    if plot_values:
+        for (i, j), v in np.ndenumerate(m):
+            ax.text(j, i, np.around(v, 2), ha='center', va='center', c='b')
+    ax.set_xlabel('Transition class', fontsize=18)
+    ax.set_ylabel('Starting class', fontsize=18)
+    plt.show()
+    if gmm_components is not None:
+        idx_ = np.unravel_index(np.argsort(m.ravel()), m.shape)
+        idx_ = np.dstack(idx_)[0][::-1]
+        print()
+        i_ = 0
+        for i in idx_:
+            if plot_toself is False and i[0] == i[1]:
+                continue
+            _, (ax1, ax2) = plt.subplots(1, 2, figsize=(fsize, fsize//2))
+            if gmm_components.shape[-1] == 3:
+                start_comp = gmm_components[states[i[0]]-1]
+                trans_comp = gmm_components[states[i[1]]-1]
+            else:
+                start_comp = np.sum(gmm_components[states[i[0]]-1], axis=-1)
+                trans_comp = np.sum(gmm_components[states[i[1]]-1], axis=-1)
+            print("Starting class  --->  Transition class (Prob: {})".
+                  format(m[tuple(i)]))
+            ax1.imshow(start_comp, cmap=cmap)
+            ax1.set_title("GMM component {}".format(states[i[0]]))
+            ax2.imshow(trans_comp, cmap=cmap)
+            ax2.set_title("GMM_component {}".format(states[i[1]]))
+            plt.show()
+            i_ = i_ + 1
+            if i_ == transitions_to_plot - 1:
+                break
+    return
+
+
+def plot_trajectories_transitions(trans_dict, k, plot_values=False, **kwargs):
+    """
+    Plots trajectory witht he associated transitions.
+
+    Args:
+        trans_dict (dict):
+            Python dictionary containing trajectories, frame numbers,
+            transitions and the averaged GMM components. Usually this is
+            an output of atomstat.transition_matrix
+        k (int): Number of trajectory to vizualize
+        plot_values (bool): Show calculated transtion rates
+        **transitions_to_plot (int):
+            number of transitions (associated with largerst prob values) to plot
+        **fsize (int): figure size
+        **cmap (str): color map
+        **fov (int or list): field of view (scan size)
+    """
+    traj = trans_dict["trajectories"][k]
+    frames = trans_dict["frames"][k]
+    trans = trans_dict["transitions"][k]
+    plot_trajectories(traj, frames, **kwargs)
+    print()
+    s_true = np.unique(traj[:, -1]).astype(np.int64)
+    plot_transitions(
+        trans, s_true, trans_dict["gmm_components"],
+        plot_values, **kwargs)
+    return
+
+
 #############################
 # Training data preparation #
 #############################
@@ -744,7 +886,7 @@ class MakeAtom:
 
 def create_lattice_mask(lattice, xy_atoms, *args, **kwargs):
     """
-    Given experimental image and xy atomic coordinates
+    Given experimental image and *xy* atomic coordinates
     creates ground truth image. Currently works only for the case
     where all atoms are one class. Notice that it will round fractional pixels.
 
@@ -858,8 +1000,8 @@ class augmentor:
     Examples:
         Suppose we have a stack of images
         and a stack of masks (aka labels aka ground truth)
-        with dimensions (n_images, height, width)
-        and (n_images, height, width, channels).
+        with dimensions :math:`n images \\times height \\times width`
+        and :math:`images \\times height \\times width \\times channels`.
         We can use the augmentor as follows.
 
         >>> # Specify size, dimensions
